@@ -8,7 +8,19 @@ use std::hash::Hash;
 /**
 Compute the shortest-path (Dijkstra) betweenness centrality for nodes.
 
-$ c_B(v) =\sum_{s,t \in V} \frac{\sigma(s, t|v)}{\sigma(s, t)} $
+# Arguments
+
+* `graph`: a [Graph](../../../struct.Graph.html) instance
+* `weighted`: set to `true` to use edge weights when computing the betweenness centrality
+* `normalized`: set to `true` to normalize the node centrality values
+
+# Examples
+
+```
+use graphrs::{algorithms::{centrality::{betweenness}}, generators};
+let graph = generators::social::karate_club_graph();
+let centralities = betweenness::betweenness_centrality(&graph, false, true);
+```
 
 # References
 
@@ -20,8 +32,8 @@ pub fn betweenness_centrality<T, A>(
     normalized: bool,
 ) -> Result<HashMap<T, f64>, Error>
 where
-    T: Hash + Eq + Copy + Ord + Display + Send + Sync,
-    A: Copy + Send + Sync,
+    T: Hash + Eq + Clone + Ord + Display + Send + Sync,
+    A: Clone + Send + Sync,
 {
     let all_pairs = dijkstra::all_pairs(graph, weighted, None, false);
     match all_pairs {
@@ -42,14 +54,12 @@ where
 
 fn add_missing_nodes_to_between_counts<T, A>(
     between_counts: &mut HashMap<T, f64>,
-    nodes: &Vec<&Node<T, A>>,
+    nodes: &[&Node<T, A>],
 ) where
-    T: Hash + Eq + Copy + Ord + Display + Send + Sync,
+    T: Hash + Eq + Clone + Ord + Display + Send + Sync,
 {
     for node in nodes {
-        if !between_counts.contains_key(&node.name) {
-            between_counts.insert(node.name, 0.0);
-        }
+        between_counts.entry(node.name.clone()).or_insert(0.0);
     }
 }
 
@@ -58,19 +68,19 @@ pub fn get_all_shortest_paths_involving<T>(
     node_name: T,
 ) -> Vec<&ShortestPathInfo<T>>
 where
-    T: Copy + Eq,
+    T: Clone + Eq,
 {
     pairs
         .values()
         .map(|x| x.values())
         .flatten()
-        .filter(|x| x.contains_path_through_node(node_name))
+        .filter(|x| x.contains_path_through_node(node_name.clone()))
         .collect()
 }
 
 fn get_between_counts<T>(pairs: &HashMap<T, HashMap<T, ShortestPathInfo<T>>>) -> HashMap<T, f64>
 where
-    T: Hash + Eq + Copy + Ord + Display,
+    T: Hash + Eq + Clone + Ord + Display,
 {
     let short_paths = pairs.values().map(|x| x.values()).flatten();
     short_paths
@@ -82,9 +92,9 @@ where
         })
 }
 
-fn get_node_counts<T>(paths: &Vec<Vec<T>>) -> Vec<(T, f64)>
+fn get_node_counts<T>(paths: &[Vec<T>]) -> Vec<(T, f64)>
 where
-    T: Copy,
+    T: Clone,
 {
     let paths_count = paths.len() as f64;
     paths
@@ -103,7 +113,7 @@ fn rescale<T>(
     directed: bool,
 ) -> HashMap<T, f64>
 where
-    T: Hash + Eq + Copy + Ord + Display,
+    T: Hash + Eq + Clone + Ord + Display,
 {
     let scale = get_scale(num_nodes, normalized, directed);
     match scale {
@@ -137,37 +147,37 @@ mod tests {
 
     #[test]
     fn test_get_node_counts_1() {
-        let result = get_node_counts::<&str>(&vec![]);
+        let result = get_node_counts::<&str>(&[]);
         assert_eq!(result, vec![]);
     }
 
     #[test]
     fn test_get_node_counts_2() {
-        let result = get_node_counts(&vec![vec!["n1", "n3"]]);
+        let result = get_node_counts(&[vec!["n1", "n3"]]);
         assert_eq!(result, vec![]);
     }
 
     #[test]
     fn test_get_node_counts_3() {
-        let result = get_node_counts(&vec![vec!["n1", "n2", "n3"]]);
+        let result = get_node_counts(&[vec!["n1", "n2", "n3"]]);
         assert_eq!(result, vec![("n2", 1.0)]);
     }
 
     #[test]
     fn test_get_node_counts_4() {
-        let result = get_node_counts(&vec![vec!["n1", "n3"], vec!["n1", "n2", "n3"]]);
+        let result = get_node_counts(&[vec!["n1", "n3"], vec!["n1", "n2", "n3"]]);
         assert_eq!(result, vec![("n2", 0.5)]);
     }
 
     #[test]
     fn test_get_node_counts_5() {
-        let result = get_node_counts(&vec![vec!["n1", "n2", "n3", "n4", "n5"]]);
+        let result = get_node_counts(&[vec!["n1", "n2", "n3", "n4", "n5"]]);
         assert_eq!(result, vec![("n2", 1.0), ("n3", 1.0), ("n4", 1.0)]);
     }
 
     #[test]
     fn test_get_node_counts_6() {
-        let result = get_node_counts(&vec![
+        let result = get_node_counts(&[
             vec!["n1", "n2", "n3", "n4", "n5"],
             vec!["n1", "n2", "n6", "n5"],
         ]);
