@@ -1,5 +1,6 @@
 use crate::{Error, ErrorKind, Graph, GraphSpecs};
-extern crate rand;
+use rand::{Rng, RngCore, SeedableRng};
+use rand_chacha::ChaCha20Rng;
 
 /**
 Returns an Erdos-Renyi or binomial random graph.
@@ -14,7 +15,7 @@ Returns an Erdos-Renyi or binomial random graph.
 
 ```
 use graphrs::{generators};
-let graph = generators::random::fast_gnp_random_graph(250, 0.25, true);
+let graph = generators::random::fast_gnp_random_graph(250, 0.25, true, None);
 assert!(graph.is_ok());
 ```
 */
@@ -22,6 +23,7 @@ pub fn fast_gnp_random_graph(
     num_nodes: i32,
     edge_probability: f64,
     directed: bool,
+    seed: Option<u64>,
 ) -> Result<Graph<i32, ()>, Error> {
     if edge_probability <= 0.0 || edge_probability >= 1.0 {
         return Err(Error {
@@ -32,15 +34,17 @@ pub fn fast_gnp_random_graph(
             ),
         });
     }
+    let mut rng = get_random_number_generator(seed);
     match directed {
-        true => fast_gnp_random_graph_directed(num_nodes, edge_probability),
-        false => fast_gnp_random_graph_undirected(num_nodes, edge_probability),
+        true => fast_gnp_random_graph_directed(num_nodes, edge_probability, &mut rng),
+        false => fast_gnp_random_graph_undirected(num_nodes, edge_probability, &mut rng),
     }
 }
 
 fn fast_gnp_random_graph_directed(
     num_nodes: i32,
     edge_probability: f64,
+    rng: &mut Box<dyn RngCore>,
 ) -> Result<Graph<i32, ()>, Error> {
     let mut graph = Graph::new(GraphSpecs::directed_create_missing());
     let mut w: i32 = -1;
@@ -48,16 +52,16 @@ fn fast_gnp_random_graph_directed(
     let mut v = 0;
     let mut edges = vec![];
     while v < num_nodes {
-        let lr: f64 = (1.0_f64 - rand::random::<f64>()).ln();
+        let lr: f64 = (1.0_f64 - rng.gen::<f64>()).ln();
         w = w + 1 + ((lr / lp) as i32);
         if v == w {
-            w = w + 1;
+            w += 1;
         }
         while v < num_nodes && num_nodes <= w {
-            w = w - num_nodes;
-            v = v + 1;
+            w -= num_nodes;
+            v += 1;
             if v == w {
-                w = w + 1;
+                w += 1;
             }
         }
         if v < num_nodes {
@@ -73,6 +77,7 @@ fn fast_gnp_random_graph_directed(
 fn fast_gnp_random_graph_undirected(
     num_nodes: i32,
     edge_probability: f64,
+    rng: &mut Box<dyn RngCore>,
 ) -> Result<Graph<i32, ()>, Error> {
     let mut graph = Graph::new(GraphSpecs::undirected_create_missing());
     let mut w: i32 = -1;
@@ -80,11 +85,11 @@ fn fast_gnp_random_graph_undirected(
     let mut v = 1;
     let mut edges = vec![];
     while v < num_nodes {
-        let lr: f64 = (1.0_f64 - rand::random::<f64>()).ln();
+        let lr: f64 = (1.0_f64 - rng.gen::<f64>()).ln();
         w = w + 1 + ((lr / lp) as i32);
         while w >= v && v < num_nodes {
-            w = w - v;
-            v = v + 1;
+            w += v;
+            v += 1;
         }
         if v < num_nodes {
             edges.push((v, w));
@@ -95,3 +100,10 @@ fn fast_gnp_random_graph_undirected(
         Ok(_) => Ok(graph),
     }
 }
+
+fn get_random_number_generator(seed: Option<u64>) -> Box<dyn RngCore> {
+    match seed {
+        None => Box::new(rand::thread_rng()),
+        Some(s) => Box::new(ChaCha20Rng::seed_from_u64(s)),
+    }
+}
